/*
 * CONTRAT DE LICENCE DE LOGICIEL LIBRE CeCILL-B
 * https://cecill.info/licences/Licence_CeCILL-B_V1-fr.html
 */
package fr.unistra.sil.erp.back.controller.api;

import com.google.gson.Gson;
import com.google.gson.JsonParseException;

import static fr.unistra.sil.erp.back.WebMvcConfig.MAPPING_SUBTRANSAC;

import fr.unistra.sil.erp.back.controller.ISubmitTransactionController;
import fr.unistra.sil.erp.back.repository.DatabaseConnectionException;
import fr.unistra.sil.erp.back.repository.DatabaseResourceNotFoundException;
import fr.unistra.sil.erp.back.repository.DatabaseUpdateException;
import fr.unistra.sil.erp.back.model.Stock;
import fr.unistra.sil.erp.back.model.Transaction;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import fr.unistra.sil.erp.back.repository.IStocksRepository;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

/**
 * REST Controller for submitting transactions.
 *
 * @author BEAUVAIS ANTOINE
 */
@RestController
public class ApiSubmitTransactionController implements ISubmitTransactionController {

    private final IStocksRepository repository;

    public ApiSubmitTransactionController(IStocksRepository repository) {
        this.repository = repository;
    }

    /**
     * Handler for transaction submissions.
     *
     * @param request  the HTTP Servlet Request provided by Spring.
     * @param response the HTTP Servlet Response provided by Spring.
     * @return the response for the user.
     * @throws ApiBadRequestException       if the query failed.
     * @throws ApiServerErrorException      Server error.
     * @throws ApiResourceNotFoundException Stock not found.
     */
    @RequestMapping(value = MAPPING_SUBTRANSAC, method = RequestMethod.POST)
    @Override
    public ResponseEntity<Object> submitTransaction(HttpServletRequest request,
                                                    HttpServletResponse response)
            throws ApiBadRequestException, ApiServerErrorException,
            ApiResourceNotFoundException {
        Gson gson = new Gson();
        String body;
        try {
            body = request.getReader().lines()
                    .collect(Collectors.joining(System.lineSeparator()));
        } catch (IOException ex) {
            Logger.getLogger(ApiSubmitTransactionController.class.getName())
                    .log(Level.SEVERE, "Unparseable body.", ex);
            throw new ApiBadRequestException("Unparseable body.");
        }
        if (body == null)
            throw new ApiBadRequestException("Missing JSON body.");

        Transaction t;
        try {
            t = gson.fromJson(body, Transaction.class);
        } catch (JsonParseException ex) {
            throw new ApiBadRequestException("Invalid JSON: syntax error.");
        }

        if (t == null)
            throw new ApiBadRequestException("Missing JSON body.");

        if (!t.checkIfValid())
            throw new ApiBadRequestException("Invalid JSON schema.");

        Stock s;
        try {
            s = repository.getStockForItem(t.getItem());
            if (s == null)
                throw new ApiServerErrorException("Database failure.");

            int newQuantity = s.getQuantity() + t.getQuantity();
            if (newQuantity < 0)
                newQuantity = 0;

            repository.updateStock(s.getId(), newQuantity);
        } catch (DatabaseConnectionException ex) {
            Logger.getLogger(ApiSubmitTransactionController.class.getName())
                    .log(Level.SEVERE, ex.getMessage(), ex);
            throw new ApiServerErrorException("Database failure.");
        } catch (DatabaseResourceNotFoundException ex) {
            throw new ApiResourceNotFoundException("No stock found for item " +
                    t.getItem());
        } catch (DatabaseUpdateException ex) {
            Logger.getLogger(ApiSubmitTransactionController.class.getName())
                    .log(Level.SEVERE, ex.getMessage(), ex);
            throw new ApiServerErrorException("Database update failure.");
        }

        return new ResponseEntity<>(t, HttpStatus.CREATED);
    }

}
