package fr.unistra.sil.erp.back.repository.stock;

import fr.unistra.sil.erp.back.repository.DatabaseConnectionException;
import fr.unistra.sil.erp.back.repository.DatabaseResourceNotFoundException;
import fr.unistra.sil.erp.back.repository.DatabaseUpdateException;
import fr.unistra.sil.erp.back.model.Stock;
import fr.unistra.sil.erp.back.repository.IStocksRepository;
import fr.unistra.sil.erp.back.repository.SqliteRepository;
import org.springframework.stereotype.Repository;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

@Repository
public class SqliteStocksRepository extends SqliteRepository implements IStocksRepository {

    /**
     * Query used to fetch all stocks stored in the database.
     */
    private static final String SQL_GETSTOCKS =
            "SELECT s.id AS id, s.item AS item, " +
                    "i.name AS name, s.quantity AS quantity FROM stocks s " +
                    "INNER JOIN items i ON s.item = i.ref";

    private static final String SQL_GETSTOCKFORITEM = SQL_GETSTOCKS +
            " WHERE item = ?";

    private static final String SQL_UPDATESTOCK =
            "UPDATE stocks SET quantity = ? WHERE item = ?";

    /**
     * Class constructor.
     */
    public SqliteStocksRepository(Connection conn) {
        super(conn);
    }

    /**
     * Retrieves all stock entries from the database.
     *
     * @return the list of stock entries.
     */
    @Override
    public List<Stock> getStocks() {
        ResultSet rs = this.query(SQL_GETSTOCKS);
        if (rs == null)
            return null;

        List<Stock> res = new ArrayList<>();
        try {
            while (rs.next()) {
                Stock s = new Stock(rs.getInt("id"),
                        rs.getInt("item"), rs.getString("name"),
                        rs.getInt("quantity"));
                res.add(s);
            }
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch results.", ex);
            return null;
        }

        return res;
    }

    /**
     * Returns the stock entry for the specified item.
     *
     * @param itemId the item's ID.
     * @return the Stock entry.
     * @throws DatabaseResourceNotFoundException if the stock doesn't exist.
     */
    @Override
    public Stock getStockForItem(int itemId)
            throws DatabaseResourceNotFoundException {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_GETSTOCKFORITEM);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }

        try {
            ps.setInt(1, itemId);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            return null;
        }

        ResultSet rs;
        try {
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch stock for item " +
                            itemId, ex);
            return null;
        }

        Stock s;
        try {
            if (rs.next())
                s = new Stock(rs.getInt("id"), rs.getInt("item"),
                        rs.getString("name"),
                        rs.getInt("quantity"));
            else
                throw new DatabaseResourceNotFoundException("Stock not found.");
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to parse results.", ex);
            return null;
        }

        return s;
    }

    /**
     * Updates the specified stock entry with the new quantity.
     *
     * @param id       the stock's ID.
     * @param quantity the associated quantity.
     * @throws DatabaseConnectionException when the connection to the DB fails.
     * @throws DatabaseUpdateException     when the update fails.
     */
    @Override
    public void updateStock(int id, int quantity)
            throws DatabaseConnectionException, DatabaseUpdateException {

        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_UPDATESTOCK);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            throw new DatabaseConnectionException("Failed to connect to DB.");
        }

        try {
            ps.setInt(1, quantity);
            ps.setInt(2, id);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            throw new DatabaseUpdateException("Failed to set query params.");
        }

        try {
            ps.execute();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName())
                    .log(Level.SEVERE, "Failed to execute query.", ex);
            throw new DatabaseUpdateException("Failed to execute query.");
        }
    }
}
